#!/usr/bin/env python

# Copyright 2011-2014 Arthur Noel
#
# This file is part of Yanc.
#
# Yanc is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Yanc is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yanc. If not, see <http://www.gnu.org/licenses/>.

import os

from setuptools import setup
from setuptools.command import test

# monkey patch test command to make nose work with `setup.py test`
# see: http://fgimian.github.io/blog/2014/04/27/running-nose-tests-with-plugins-using-the-python-setuptools-test-command/
test._test = test.test


class NoseTestCommand(test._test):

    user_options = test._test.user_options + [
        ("args=", "a", "Arguments to pass to nose"),
    ]

    def initialize_options(self):
        test._test.initialize_options(self)
        self.args = None

    def finalize_options(self):
        test._test.finalize_options(self)
        self.args = self.args and self.args.strip().split() or []
        self.test_suite = True

    def run_tests(self):
        import nose
        nose.run_exit(argv=["nosetests"] + self.args)

test.test = NoseTestCommand

setup(
    name="yanc",
    version="0.3.3",
    description="Yet another nose colorer",
    long_description=open(
        os.path.join(os.path.dirname(__file__), "README.rst")).read(),
    license="GPL",
    keywords="nose color",
    author="Arthur Noel",
    author_email="arthur@0compute.net",
    url="https://github.com/0compute/yanc",
    packages=("yanc",),
    tests_require=[
        "nose",
    ],
    test_suite="nose.collector",
    entry_points = {
        "nose.plugins": ("yanc=yanc.yancplugin:YancPlugin",),
    },
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Software Development :: Testing",
    ],
)



# Copyright 2011-2014 Arthur Noel
#
# This file is part of Yanc.
#
# Yanc is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Yanc is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yanc. If not, see <http://www.gnu.org/licenses/>.

import re


class ColorStream(object):

    COLOR_NAMES = ["grey", "red", "green", "yellow", "blue", "magenta", "cyan",
                   "white"]
    COLOR_CODES = dict(zip(COLOR_NAMES, list(range(30, 38))))
    COLOR_FMT = "\033[%dm%s\033[0m"

    COLORS = dict(
        green=("OK", "ok", "."),
        red=("ERROR", "FAILED", "errors", "E"),
        yellow=("FAILURE", "FAIL", "failures", "F"),
        magenta=("SKIP", "S"),
        blue=("-" * 70, "=" * 70),
    )

    def __init__(self, stream):
        self._stream = stream
        self._color_map = {}
        self._patten_map = {}
        for color, labels in self.COLORS.items():
            for label in labels:
                self._color_map[label] = color
                if len(label) > 1:
                    self._patten_map[label] = re.compile(r"%s=\d+" % label)

    def __getattr__(self, key):
        return getattr(self._stream, key)

    def _color_wrap(self, string, color):
        return self.COLOR_FMT % (self.COLOR_CODES[color], string)

    def _colorize(self, string, color=None):
        if string:
            if color is None:
                color = self._color_map.get(string)
                if color is None:
                    for key in self._color_map:
                        # looking for a test failure as LABEL: str(test)
                        if string.startswith(key + ":"):
                            segments = string.split(":")
                            label = self._colorize(segments[0] + ":",
                                                   self._color_map[key])
                            desc = ":".join(segments[1:])
                            return label + desc
                    for key, key_color in self._color_map.items():
                        # looking for label=number in the summary
                        pattern = self._patten_map.get(key)
                        if pattern is not None:
                            for match in pattern.findall(string):
                                string = string.replace(
                                    match, self._colorize(match, key_color)
                                )
            if color is not None:
                string = self._color_wrap(string, color)
        return string

    def write(self, string):
        self._stream.write(self._colorize(string))

    def writeln(self, string=""):
        self._stream.writeln(self._colorize(string))



# Copyright 2011-2014 Arthur Noel
#
# This file is part of Yanc.
#
# Yanc is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Yanc is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yanc. If not, see <http://www.gnu.org/licenses/>.

from nose.plugins import Plugin

from yanc.colorstream import ColorStream


class YancPlugin(Plugin):
    """Yet another nose colorer"""

    name = "yanc"

    def options(self, parser, env):
        super(YancPlugin, self).options(parser, env)
        parser.add_option(
            "--yanc-color",
            action="store",
            dest="yanc_color",
            default=env.get("NOSE_YANC_COLOR"),
            help="YANC color override - one of on,off [NOSE_YANC_COLOR]",
        )

    def configure(self, options, conf):
        super(YancPlugin, self).configure(options, conf)
        if options.yanc_color is None and not conf.worker \
                and hasattr(conf.stream, "isatty") and conf.stream.isatty():
            # if color is not set then set color on the basis of the stream's
            # tty status - this is set on options so that the value is
            # propagated to multiprocess workers meaning that the option is
            # never None when conf.worker is True
            # XXX: apparently not, see #6
            options.yanc_color = "on"
        self.color = options.yanc_color != "off"

    def setOutputStream(self, stream):
        # when run in series, this method gets called once and that is enough,
        # when run in parallel, this method is called at the top level which
        # deals with the test summary information but the workers need
        # prepareTestResult to have their output colored
        return self.color and ColorStream(stream) or stream

    def prepareTestResult(self, result):
        if not isinstance(result.stream, ColorStream):
            result.stream = self.setOutputStream(result.stream)



# Copyright 2011-2014 Arthur Noel
#
# This file is part of Yanc.
#
# Yanc is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Yanc is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Yanc. If not, see <http://www.gnu.org/licenses/>.

import difflib

import re
import unittest

from nose.plugins import PluginTester

try:
    import unittest2
except ImportError:
    pass
else:  # pragma: no cover - unittest shim for those that don't have it
    import sys
    sys.modules["unittest"] = unittest2

from yanc.yancplugin import YancPlugin


EXCEPTION_PLAIN = """EF
======================================================================
ERROR: runTest (yanc.test_yanc.TC1)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test_yanc.py", line 1, in runTest
    raise ValueError("xxx")
ValueError: xxx

======================================================================
FAIL: runTest (yanc.test_yanc.TC2)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test_yanc.py", line 1, in runTest
    self.assertTrue(False)
AssertionError

----------------------------------------------------------------------
Ran 2 tests in 0.001s

FAILED (errors=1, failures=1)
"""

EXCEPTION_COLOR = """\x1b[31mE\x1b[0m\x1b[33mF\x1b[0m
\x1b[34m======================================================================\x1b[0m
\x1b[31mERROR:\x1b[0m runTest (yanc.test_yanc.TC1)
\x1b[34m----------------------------------------------------------------------\x1b[0m
Traceback (most recent call last):
  File "test_yanc.py", line 1, in runTest
    raise ValueError("xxx")
ValueError: xxx

\x1b[34m======================================================================\x1b[0m
\x1b[33mFAIL:\x1b[0m runTest (yanc.test_yanc.TC2)
\x1b[34m----------------------------------------------------------------------\x1b[0m
Traceback (most recent call last):
  File "test_yanc.py", line 1, in runTest
    self.assertTrue(False)
AssertionError

\x1b[34m----------------------------------------------------------------------\x1b[0m
Ran 2 tests in 0.001s

\x1b[31mFAILED\x1b[0m (\x1b[31merrors=1\x1b[0m, \x1b[33mfailures=1\x1b[0m)
"""

FILE_PATTERN = re.compile('File "[^"]+", line \d+')
TIME_PATTEN = re.compile("\d+\.\d+s")
ASSERTIONERROR_PATTERN = re.compile("AssertionError.*")


class _TestYanc(PluginTester, unittest.TestCase):

    activate = '--with-yanc'

    plugins = [YancPlugin()]

    exc_str = EXCEPTION_COLOR

    def makeSuite(self):

        class TC1(unittest.TestCase):
            def runTest(self):
                raise ValueError("xxx")

        class TC2(unittest.TestCase):
            def runTest(self):
                self.assertTrue(False)

        return unittest.TestSuite([TC1(), TC2()])

    def test_yanc(self):
        exc_str = FILE_PATTERN.sub('File "test_yanc.py", line 1',
                                   str(self.output))
        exc_str = TIME_PATTEN.sub("0.001s", exc_str)
        exc_str = ASSERTIONERROR_PATTERN.sub("AssertionError", exc_str)
        if self.exc_str != exc_str:  # pragma: no cover - only hit on fail
            diff = difflib.unified_diff(self.exc_str.splitlines(True),
                                        exc_str.splitlines(True),
                                        "expected", "actual")
            self.fail("".join(diff))


class TestYancColorAuto(_TestYanc):
    args = ('--yanc-color=on',)


class TestYancColorOn(_TestYanc):
    args = ('--yanc-color=on',)


class TestYancColorOff(_TestYanc):
    args = ('--yanc-color=off',)
    exc_str = EXCEPTION_PLAIN
